' Region Description
'
' Name: Get-FolderStructure.vbs
' Author: Nils Kaczenski, nka@michael-wessel.de
' Version: 1.0 (160906)
' Description:
'   Reads a folder structure recursively and writes it to a CSV file
'   Includes full folder path, path length, and displays subfolders at the correct level
'	The report file is stored in the user's Documents folder   
'	The CSV file is best used With Excel
'   Execute this for local paths only as network paths will take too long
' EndRegion


' Const
Const TristateTrue = -1
Const ForReading = 1
' EndConst

strTitle = "Get-FolderStructure.vbs"

dtmStart = Now

' Create objects
Set objFSO = CreateObject("Scripting.FileSystemObject")
Set objShell = CreateObject("WScript.Shell")

' build output path
strDocuments = objShell.ExpandEnvironmentStrings("%USERPROFILE%") & "\Documents"

' ask for root folder to analyze
strRoot = InputBox("Welches ist der Startordner?", strTitle, strDocuments)

' build ISO date for file name
strMyDate = Year(Date) & Right("0" & Month(Date), 2) & Right("0" & Day(Date), 2) & "-" & Right("0" & Hour(Now), 2) & Right("0" & Minute(Now), 2) & Right("0" & Second(Now), 2)
' escape root folder name to use in output file name
strRootEscaped = Replace(Replace(strRoot, "\", "_"), ":", "_")
' access root folder, make sure it exists
On Error Resume Next
Set objRoot = objFSO.GetFolder(strRoot)
If Err.number <> 0 Then
	MsgBox strRoot & vbNewLine & "existiert nicht",vbOK + vbCritical, strTitle
	WScript.Quit
End If
On Error Goto 0

' build output file names and create work file
strOutputFileName = objFSO.BuildPath(strDocuments, "FolderStructure-" & strRootEscaped & "-" & strMyDate & ".work.txt")
strOutputResult = objFSO.BuildPath(strDocuments, "FolderStructure-" & strRootEscaped & "-" & strMyDate & ".txt")
Set objOutput = objFSO.CreateTextFile(strOutputFileName,vbTrue,vbTrue)

' initialize counters
intMaxFolderDepth = 0
intFolderDepth = 0
intFolderCount = 0

' start walking the folder structure
GetFolder objRoot
objOutput.Close
Set objOutput = Nothing

' build CSV header line
strHeader = "FolderNo,PathLength,FullPath,FolderDepth"
For i = 1 To intMaxFolderDepth - 1
	strHeader = strHeader & ",Folder" & i
Next
' open final result file and write header
Set objOutputResult = objFSO.CreateTextFile(strOutputResult,vbTrue,vbTrue)
objOutputResult.WriteLine strHeader
' open work file and copy contents into result file line by line
Set objInput = objFSO.GetFile(strOutputFileName).OpenAsTextStream(ForReading,TristateTrue)
Do Until objInput.AtEndOfStream
	strLine = objInput.ReadLine
	objOutputResult.WriteLine strLine
Loop
objInput.Close
objFSO.DeleteFile strOutputFileName
dtmEnd = Now

' output file name in an input box so the user can easily copy it
strMessage = "Fertig. " & intFolderCount & " Ordner ausgegeben." & vbNewLine _
	& "Startzeit: " & dtmStart & vbNewLine _
	& "Endzeit: " & dtmEnd & vbNewLine & vbNewLine _
	& "Dateiname (Kopieren mit Strg-C):"
InputBox strMessage,strTitle,strOutputResult

' uncomment the following line to automatically open the report file
objShell.Exec "notepad " & strOutputResult

Sub GetFolder(objStartFolder)
	' increment level counter
	intFolderDepth = intFolderDepth + 1
	' increment overall folder counter
	intFolderCount = intFolderCount + 1

	' check if this is a new maximum level
	if intFolderDepth > intMaxFolderDepth then intMaxFolderDepth = intFolderDepth
	
	' if there are no subfolders quit this recursion
	On Error Resume Next
	If objStartFolder.Subfolders.Count = 0 Then
		intFolderDepth = intFolderDepth - 1
		On Error Goto 0
		Exit Sub
	End If
	' walk through the subfolders and get data
	For Each objSubfolder In objStartFolder.Subfolders
		strOutput = intFolderCount
		strOutput = strOutput & "," & Len(objSubfolder.Path)
		strOutput = strOutput & ",""" & objSubfolder.Path & """," & intFolderDepth
		strOutput = strOutput & String(intFolderDepth,",")
		strOutput = strOutput & """" & objSubfolder.Name & """"
		' uncomment the following line to get screen output (only when running with cscript!)
		' WScript.Echo strOutput
		' write to work file - this saves memory
		objOutput.WriteLine strOutput
		' now recurse through all subfolders of this folders
		GetFolder objSubfolder
		On Error Goto 0
	Next
	' decrease level counter and finish this round
	intFolderDepth = intFolderDepth - 1
End Sub
